#include "support_common.h" /* include peripheral declarations and more */

uint32 i2c_init(uint32 arg);
uint32 i2c_send(uint32 arg);

asm void jump_table(void) {
  jmp i2c_init      // Driver #0   
  jmp i2c_send      // Driver #1   
}

asm void SystemSleep(void) {
  move 0x210,a0
  jmp (a0)
}

uint32 i2c_init(uint32 arg) {
  #pragma unused(arg)
  // Configure ports.
  MCF_GPIO_PQSPAR =
    MCF_GPIO_PQSPAR_QSPI_CLK_SCL |
    MCF_GPIO_PQSPAR_QSPI_CS0_SDA;
  // Specify I2C clock rate.
  MCF_I2C_I2FDR =
    MCF_I2C_I2FDR_IC(0x38); // (x640)
  // Configure I2C module
  MCF_I2C_I2CR =
    MCF_I2C_I2CR_MTX |  // Transmit
    MCF_I2C_I2CR_IEN;   // I2C enable
}

uint32 i2c_send(uint32 arg) {
  uint8  address = ((uint8*)arg)[0];
  uint8  command = ((uint8*)arg)[1];
  uint8  data    = ((uint8*)arg)[2];

  while (MCF_I2C_I2SR & MCF_I2C_I2SR_IBB) {
    SystemSleep(); // Wait for BUS idle
  }
  MCF_I2C_I2SR = 0x00; // Clear flags
  MCF_I2C_I2CR |= MCF_I2C_I2CR_MSTA; // START
  do {
    MCF_I2C_I2DR = (uint8)(address << 1); // WRITE for 0100000
    while (!(MCF_I2C_I2SR & MCF_I2C_I2SR_IIF)) {
      SystemSleep(); // Wait for IIF
    }
    if (MCF_I2C_I2SR & MCF_I2C_I2SR_RXAK) {
      break; // Quit if no ACK
    }
    MCF_I2C_I2SR = 0x00; // Clear flags
    MCF_I2C_I2DR = command; // command code
    while (!(MCF_I2C_I2SR & MCF_I2C_I2SR_IIF)) {
      SystemSleep(); // Wait for IIF
    }
    if (MCF_I2C_I2SR & MCF_I2C_I2SR_RXAK) {
      break; // Quit if no ACK
    }
    MCF_I2C_I2SR = 0x00; // Clear flags
    MCF_I2C_I2DR = data; // data to be sent
    while (!(MCF_I2C_I2SR & MCF_I2C_I2SR_IIF)) {
      SystemSleep(); // Wait for IIF
    }
  } while (0);
  MCF_I2C_I2CR &= ~MCF_I2C_I2CR_MSTA; // STOP
}
